@@ -1,5 +1,7 @@
 from .version import __version__
+from time import gmtime, strftime
 print (f'(Running XClone {__version__})')
+print (strftime("%Y-%m-%d %H:%M:%S", gmtime()))
 
 # set simplified alias
 from . import preprocessing as pp
@@ -12,4 +14,4 @@ from .model import phasing as phasing
 from .model import mixture as mixture
 
 from ._logging import get_logger
-from ._config import XCloneConfig, settings
\ No newline at end of file
+from ._config import PreprocessingConfig, XCloneConfig, settings
\ No newline at end of file
