@@ -5,5 +5,7 @@ from .demo_datasets._datasets import (
     tnbc1_rdr,
     tnbc1_baf,
     bch869_rdr,
-    bch869_baf
+    bch869_baf,
+    bch869_baf_50kb,
+    gx109_rdr
 )
\ No newline at end of file
