@@ -28,4 +28,11 @@ def get_KNN_connectivities_from_expr(Xdata, expr_adata):
         Xdata.obsp["connectivities_expr"] = expr_adata.obsp["connectivities"].copy()
     else:
         raise ValueError("[XClone] Xdata cell order are not matched! Pls check and do preprocessing!")
-    return Xdata
\ No newline at end of file
+    return Xdata
+
+def BAF_prob_correct(Xdata, Xlayer1, Xlayer2):
+    """
+    todo: maybe add function to use 3 states layer 
+    to correct 5 states layer BAF prob noise.
+    """
+    pass
\ No newline at end of file
