@@ -20,9 +20,9 @@ from .HMM_NB import calculate_Xemm_prob2
 from .HMM_NB import calculate_Xemm_probTry
 from .HMM_base import XHMM_smoothing
 
-from ..plot._visualize  import convert_res_ann
 from ..plot._data import reorder_data_by_cellanno
 from ..plot._base_xanndata import Xheatmap
+from ..plot.CNV_plot import CNV_visualization
 
 def gene_exp_group(Xdata, n_group = 5, ref_log = True, verbose=True):
     """
@@ -162,7 +162,10 @@ def generate_emm_GT(Xdata,
 ## 1) strategy1: GMM mixture model, not good for sparse matrix.(log ratio)
 ## 2) strategy2: learn outliers (from log ratio) as init CNV states
 
-def guide_CNV_chrs(Xdata, Xlayer = "RDR_smooth", anno_key = "chr_arm"):
+def guide_CNV_chrs(Xdata, 
+                   Xlayer = "RDR_smooth", 
+                   anno_key = "chr_arm", 
+                   remove_XY = True):
     """
     Function:
     --------
@@ -178,13 +181,21 @@ def guide_CNV_chrs(Xdata, Xlayer = "RDR_smooth", anno_key = "chr_arm"):
     
     mtx_use = Xdata.layers[Xlayer]
     chr_use = Xdata.var[anno_key].unique()
-    
+
     chr_dict = {}
     for chr_ in chr_use:
         flag_ = Xdata.var[anno_key] == chr_
         avg_ = mtx_use[:, flag_].mean()
+        # avg_ = np.median(mtx_use[:, flag_])
         chr_dict[chr_] = avg_
-        
+    
+    # remove chrX+Y from the chr_dict if exists.
+    if remove_XY:
+        for chr_items in ['X', 'Y', 'Xp', 'Xq', 'Yp', 'Yq']:
+            try:
+                chr_dict.pop(chr_items)
+            except:
+                pass    
     
     sort_chr_dict = OrderedDict(sorted(chr_dict.items(), key=lambda t: t[1]))
     
@@ -198,12 +209,12 @@ def guide_CNV_chrs(Xdata, Xlayer = "RDR_smooth", anno_key = "chr_arm"):
     
     chr_lst = [loss_, neutral_, gain_]
     
-    Xdata.uns["sort_chr_dict"] = sort_chr_dict
+    Xdata.uns["chr_dict"] = dict(sort_chr_dict)
     Xdata.uns["guide_CNV_chrs_use_layers"] = Xlayer
     Xdata.uns["guide_CNV_chrs_use_anno_key"] = anno_key
     print("[XClone] RDR CNV states chrs guiding(copy loss, copy neutral, copy gain):", chr_lst)
     
-    return chr_lst, anno_key
+    return chr_lst
 
 def correct_guide_RDR_cnv(guide_cnv_ratio, threshold = 0.15):
     """
@@ -218,6 +229,17 @@ def correct_guide_RDR_cnv(guide_cnv_ratio, threshold = 0.15):
     if guide_cnv_ratio[2] - guide_cnv_ratio[1] < threshold:
         guide_cnv_ratio[2] = 1.5
         print("correct RDR CNV guiding copy gain ratio")
+    
+    ## add clipping for this version
+    if guide_cnv_ratio[0] < 0.3:
+        guide_cnv_ratio[0] = 0.3
+        print("[XClone] warning: correct RDR CNV guiding copy loss ratio")
+        print("[XClone] hints: can change guide_qt_lst")
+    if guide_cnv_ratio[2] > 3:
+        print("[XClone] warning: correct RDR CNV guiding copy gain ratio")
+        print("[XClone] hints: can change guide_qt_lst")
+        guide_cnv_ratio[2] = 3
+    
     return guide_cnv_ratio
 
 def guide_CNV_states(RDR_Xdata, 
@@ -259,6 +281,61 @@ def guide_CNV_states(RDR_Xdata,
     print("[XClone] RDR CNV states ratio guiding(copy loss, copy neutral, copy gain):", guide_cnv_ratio)
     return guide_cnv_ratio
 
+from sklearn.mixture import GaussianMixture
+def guide_RDR_CNV_ratio(RDR_Xdata, 
+                        Xlayer = "RDR_smooth", 
+                        anno_key = "chr_arm", 
+                        chr_lst = ["18p", "9q", "8q"], 
+                        states = ["CNV loss: ", "CNV neutral: ", "CNV gain: "],
+                        means_init = None,
+                        n_components = 3,
+                        max_iter = 50,
+                        **kwargs):
+    """
+    based on GMM model.
+    """
+    import matplotlib.pylab as plt
+    guide_cnv_ratio = []
+    
+    # for i, tmp_state in zip(range(len(states)), states):
+    if True:
+        
+        # flag_ = RDR_Xdata.var[anno_key] == chr_lst[i]
+        flag_ = RDR_Xdata.var[anno_key].isin(chr_lst)
+
+        data_ = RDR_Xdata[:, flag_].layers[Xlayer].reshape(-1,1)
+        
+        r = np.random.RandomState(seed=1234)
+        params = {}
+        params["n_components"] = n_components
+        params["max_iter"] = max_iter
+        params["random_state"] = r
+        params["tol"] = 1e-9
+        if means_init is not None:
+            params["means_init"] = means_init
+        # if i == 0:
+        #     params["means_init"] = np.array([0.5]).reshape(-1, 1)
+        # if i == 1:
+        #     params["means_init"] = np.array([1]).reshape(-1, 1)
+        # if i == 2:
+        #     params["means_init"] = np.array([1.5]).reshape(-1, 1)
+
+
+        params.update(**kwargs)
+        gmm = GaussianMixture(**params).fit(data_)
+
+        
+
+        sort_means = np.exp(np.sort(gmm.means_[:,0]))
+        print(sort_means)
+        # print(tmp_state, sort_means)
+        # if i == 0:
+        #     guide_cnv_ratio.append(sort_means[0])
+
+    return None
+
+
+
 def fit_CNV_ratio(Xdata, 
                   init_prob_layer = "emm_prob_log", 
                   hard_assign = True, 
@@ -520,24 +597,27 @@ def combine_BAF_emmprob(Xdata, BAF_Xdata):
     return Xdata
 
 def CNV_optimazation(Xdata,
-                    dispersion_key = "dispersion_capped",
-                    init_state_ratio = np.array([0.5, 1.0, 1.5]), 
-                    max_iter=20, 
-                    min_iter=3, 
-                    epsilon_conv=1e-2, 
-                    verbose= True,
-                    fitCNV_verbose = False,
-                    HMM_verbose = False,
-                    start_prob = None,
-                    trans_prob = None,
-                    combine_BAF = False,
-                    states_weight = np.array([1,2,3]),
-                    weights= True,
-                    nproc = 1,
-                    log_save = False, 
-                    **kwargs):
+                     depth_key = "library_ratio_capped",
+                     dispersion_key = "dispersion_capped",
+                     init_state_ratio = np.array([0.5, 1.0, 1.5]), 
+                     max_iter=20, 
+                     min_iter=3, 
+                     epsilon_conv=1e-2, 
+                     verbose= True,
+                     fitCNV_verbose = False,
+                     HMM_verbose = False,
+                     HMM_brk = "chr_arm",
+                     start_prob = None,
+                     trans_prob = None,
+                     combine_BAF = False,
+                     states_weight = np.array([1,2,3]),
+                     weights= True,
+                     nproc = 1,
+                     log_save = False, 
+                     **kwargs):
     """
-    Function:(todo func)
+    Function:
+    ---------
     iterations of fit_CNV_ratio, the loglikelihood can be used to determine the terminations.
 
     step1: get the emm_prob_log
@@ -545,11 +625,6 @@ def CNV_optimazation(Xdata,
     step3: calculate the log liklihood
     step4: determine the terminations
 
-    # init_CNV_ratio = np.array([[0.5,1.0,1.5],
-    #                  [0.5,1.0,1.5],
-    #                  [0.5,1.0,1.5],
-    #                  [0.5,1.0,1.5],
-    #                  [0.5,1.0,1.5]])
 
     Parameters:
     ----------
@@ -600,6 +675,7 @@ def CNV_optimazation(Xdata,
 
             fit_cnv_params['random_seed'] = 2
             fit_cnv_params['verbose'] = fitCNV_verbose
+            fit_cnv_params['depth_key'] = depth_key
 
             fit_cnv_params.update(**kwargs)
             CNV_ratio, model_results = fit_CNV_ratio(**fit_cnv_params)
@@ -612,7 +688,7 @@ def CNV_optimazation(Xdata,
             Xdata = calculate_Xemm_probTry(Xdata, dispersion_key, 
                                                   states = specific_states, combine_BAF = combine_BAF)
         ### posterior_mtx_log
-        update_Xdata = XHMM_smoothing(Xdata, emm_inlayer = "emm_prob_log", 
+        update_Xdata = XHMM_smoothing(Xdata, brk = HMM_brk, emm_inlayer = "emm_prob_log", 
                                       start_prob = start_prob, trans_prob = trans_prob, 
                                       nproc = nproc, verbose = HMM_verbose)
 
@@ -622,16 +698,20 @@ def CNV_optimazation(Xdata,
         _logLik = cal_log_lik(update_Xdata.layers["emm_prob_log"], update_Xdata.layers["posterior_mtx_log"])
         Logliklihood[it] = _logLik
 
-        if it > min_iter:
+        if it >= min_iter:
             if Logliklihood[it] < Logliklihood[it - 1]:
                 if verbose:
                     print("[XClone] Warning: Lower bound decreases!\n")
+                    print("Step %d, loglik decrease from %.2e to %.2e" 
+                          %(it, Logliklihood[it-1], Logliklihood[it]))
+            elif Logliklihood[it] - Logliklihood[it - 1] <= epsilon_conv:
+                break
             elif it == max_iter - 1:
                 if verbose:
-                    print("[XClone] Warning: CNV ration optimization did not converge!\n")
-                    print("[XClone] Notes: try to increase the max_iter: ", max_iter, "!\n")
-            elif Logliklihood[it] - Logliklihood[it - 1] < epsilon_conv:
-                break
+                    if max_iter > 10:
+                        print("[XClone] Warning: CNV ratio optimization did not converge!\n")
+                        print("[XClone] Notes: try to increase the max_iter: ", max_iter, "!\n")
+    
     Logliklihood = Logliklihood[:it+1]
 
     if verbose == True:
@@ -647,169 +727,4 @@ def CNV_optimazation(Xdata,
     ## Results
     update_Xdata.uns["CNV_ratio"] = CNV_ratio_dic
     update_Xdata.uns["Logliklihood"] = Logliklihood
-    return update_Xdata
-
-
-def CNV_optimazation2(Xdata, 
-                    init_state_ratio = np.array([0.5, 1.0, 1.5]), 
-                    max_iter=20, 
-                    min_iter=3, 
-                    epsilon_conv=1e-2, 
-                    verbose=True,  
-                    nproc = 1,
-                    log_save = False, 
-                    **kwargs):
-    """
-    Function:(todo func)
-    iterations of fit_CNV_ratio, the loglikelihood can be used to determine the terminations.
-
-    step1: get the emm_prob_log
-    step2: get the posterior_mtx_log
-    step3: calculate the log liklihood
-    step4: determine the terminations
-
-    # init_CNV_ratio = np.array([[0.5,1.0,1.5],
-    #                  [0.5,1.0,1.5],
-    #                  [0.5,1.0,1.5],
-    #                  [0.5,1.0,1.5],
-    #                  [0.5,1.0,1.5]])
-
-    Parameters:
-    ----------
-    Xdata: anndata.
-    init_prob:
-
-    **kwargs: parameters for fit_CNV_ratio.
-
-    Return:
-    ------
-    
-    Example:
-    -------
-
-    """
-    ## time stamp
-    start_time_ = datetime.datetime.now()
-
-    CNV_state_num = len(init_state_ratio)
-
-    try:
-        gene_group_num = len(Xdata.uns["group_genes"]) # len(ref_obs_ad1.uns["group_genes"])
-    except:
-        raise ValueError("[XClone] Xdata do not contain group genes! Pls check and do preprocessing!")
-
-    ## initalization
-    ### CNV ratio init for different gene groups
-    CNV_ratio_dic = {}
-    CNV_ratio_dic[str(0)] = np.tile(init_state_ratio, gene_group_num).reshape(-1, CNV_state_num)
-    ### likelihood init
-    Logliklihood = np.zeros(max_iter)
-
-    # iteration
-    for it in range(max_iter):
-        if verbose == True:
-            print("[XClone] CNV_optimazation iteration: ", it+1)
-
-        if it == 0: # init round
-            # specific_states = xclone.model.gene_specific_states(ref_obs_ad2_test, init_CNV_ratio)
-            specific_states = gene_specific_states(Xdata, CNV_ratio_dic[str(0)])
-            emm_prob_log = calculate_Xemm_prob2(Xdata, 
-                                            states = specific_states,
-                                            gene_specific = True, overdispersion = Xdata.var["dispersion"],
-                                            ref_normalization_term = 1,
-                                            obs_normalization_term = Xdata.obs["library_ratio"][1:])
-        else:
-            ## params setting and fit_CNV_ratio
-            fit_cnv_params = {}
-            fit_cnv_params['Xdata'] = Xdata
-            fit_cnv_params['init_prob'] = normalize(np.exp(emm_prob_log))
-            fit_cnv_params['random_seed'] = 2
-            fit_cnv_params['verbose'] = verbose
-            fit_cnv_params['feature_X'] = Xdata.uns["NMF_pred_obs"]
-
-
-            fit_cnv_params.update(**kwargs)
-            # CNV_ratio,  model_results = xclone.model.fit_CNV_ratio(ref_obs_ad2_test, xclone.model.normalize(np.exp(emm_prob_log)), 
-            # n_sample_cells = 10, dispersion_set = None, random_seed = 2, verbose=True)
-            # CNV_ratio, model_results = fit_CNV_ratio(**fit_cnv_params)
-            CNV_ratio, ref_bio_ratio, model_results = fit_CNV_ratio(**fit_cnv_params)
-
-            ## check CNV ratio
-            CNV_ratio_update  = check_CNV_ratio(CNV_ratio, CNV_ratio_dic[str(it-1)])
-            CNV_ratio_dic[str(it)] = CNV_ratio_update
-
-            specific_states = gene_specific_states(Xdata, CNV_ratio_update)
-            emm_prob_log = calculate_Xemm_prob2(Xdata, 
-                                            states = specific_states,
-                                            gene_specific = True, overdispersion = Xdata.var["dispersion"],
-                                            ref_normalization_term = 1,
-                                            obs_normalization_term = Xdata.obs["library_ratio"][1:])
-
-        ### posterior_mtx_log
-        update_Xdata = XHMM_smoothing(Xdata, emm_prob_log = emm_prob_log, nproc = nproc)
-
-        if log_save == True:
-            CNV_visualization(update_Xdata)
-
-        # check emm_prob_log and res_log shape.
-        _logLik = cal_log_lik(update_Xdata.uns["emm_prob_log"], update_Xdata.uns["posterior_mtx_log"])
-        # _logLik = cal_log_lik(emm_prob_log, posterior_mtx_log)
-        Logliklihood[it] = _logLik
-
-        if it > min_iter:
-            if Logliklihood[it] < Logliklihood[it - 1]:
-                if verbose:
-                    print("[XClone] Warning: Lower bound decreases!\n")
-            elif it == max_iter - 1:
-                if verbose:
-                    print("[XClone] Warning: CNV ration optimization did not converge!\n")
-                    print("[XClone] Notes: try to increase the max_iter: ", max_iter, "!\n")
-            elif Logliklihood[it] - Logliklihood[it - 1] < epsilon_conv:
-                break
-    Logliklihood = Logliklihood[:it+1]
-
-    if verbose == True:
-        print("iteration_end_round: ", it+1)
-        print("Logliklihood: ", Logliklihood)
-    
-    ## time stamp
-    end_time_ = datetime.datetime.now()
-    time_used = end_time_ - start_time_
-    print("Time used", time_used.seconds, "seconds")
-    
-    ## Results
-    update_Xdata.uns["CNV_ratio"] = CNV_ratio_dic
-    update_Xdata.uns["Logliklihood"] = Logliklihood
-    return update_Xdata
-
-## Visualization part in CNVratio estimation performance
-def CNV_visualization(update_Xdata, uns_key = "posterior_mtx", states_weight = np.array([1,2,3]), weights = True):
-    """
-    will be removed in this submodule.
-    have been removed to plot module.
-    default using .uns["posterior_mtx"] for visualization
-
-    deprecated:2022-07-28
-    """
-    from seaborn import palettes
-    # https://jiffyclub.github.io/palettable/
-    import palettable
-    import seaborn as sns
-    ## transfer data to anndata for visualization
-    res_cnv_ad, res_cnv_weights_ad1 = convert_res_ann(update_Xdata.uns[uns_key], update_Xdata, states_weight)
-    if weights:
-        center_value = 2.0
-        ## reorder the cells based on annotation
-        res_cnv_weights_ad1_re = reorder_data_by_cellanno(res_cnv_weights_ad1, cell_anno_key="cell_type")
-        # Xheatmap(res_cnv_weights_ad1_re, cell_anno_key = 'cell_type', center=True) ## error because True equals value 1
-        Xheatmap(res_cnv_weights_ad1_re, cell_anno_key = 'cell_type', center = center_value)
-    else:
-        # category visualization
-        center_value = float(len(states_weight)//2)
-        res_cnv_ad_re = reorder_data_by_cellanno(res_cnv_ad, cell_anno_key="cell_type")
-        Xheatmap(res_cnv_ad_re, cell_anno_key = 'cell_type', center=center_value, cmap = palettable.colorbrewer.diverging.PuOr_5.mpl_colors)
-        # Xheatmap(res_cnv_ad_re, cell_anno_key = 'cell_type', center=True, cmap = sns.color_palette("hls", 6))
-        # Xheatmap(res_cnv_ad_re, cell_anno_key = 'cell_type', center=True, cmap = sns.color_palette("Set2"))
-        # Xheatmap(res_cnv_ad_re, cell_anno_key = 'cell_type', center=True, cmap = palettable.colorbrewer.diverging.PiYG_5.mpl_colors)
-        # Xheatmap(res_cnv_ad_re, cell_anno_key = 'cell_type', center=True, cmap = palettable.cartocolors.diverging.ArmyRose_5.mpl_colors)
-        
\ No newline at end of file
+    return update_Xdata
\ No newline at end of file
