@@ -1,4 +1,5 @@
-"""[XClone] Base plotting functions for AnnData.
+"""[XClone] 
+Base heatmap plotting functions for XClone CNV calling.
 """
 # Author: Rongting Huang
 # rthuang@connect.hku.hk
@@ -100,7 +101,9 @@ def set_Xheatmap_args(legend_pos=2):
     paras['center'] = False
     paras['rasterized'] = True
     ## figure setting
-    paras['figsize'] = (16,8)
+    paras['figsize'] = (16,8) # publication
+    ### webplot
+    # paras['figsize'] = (12,8)
 
     ## legend
     legend_paras = {}
@@ -121,9 +124,10 @@ def Xheatmap(
     change_colorbar: bool = True,
     colorbar_ticks = [0, 1, 2], 
     colorbar_label = ["copy loss", "copy neutral", "copy gain"], 
-    colorbar_name = "CNV_states",
+    colorbar_name = "CNV states",
     set_grid: bool = False, 
     save_file: bool = False,
+    set_dpi = 150,
     out_file = 'XClone_Xheatmap_test.pdf',
     **kwargs):
     """
@@ -223,7 +227,7 @@ def Xheatmap(
     if save_file == False:
         plt.show()
     else:
-        plt.savefig(out_file, bbox_inches='tight', dpi=600)
+        plt.savefig(out_file, bbox_inches='tight', dpi=set_dpi)
         plt.show()
         plt.close()
 
@@ -244,7 +248,10 @@ def set_XXheatmap_args(legend_pos=2, legend_mode=1):
     paras['center'] = False
     paras['rasterized'] = True
     ## figure setting
-    paras['figsize'] = (12,8)
+    paras['figsize'] = (16,8) # publication
+    ### webplot
+    # paras['figsize'] = (12,8)
+
 
     ## legend
     legend_paras = {}
@@ -305,8 +312,9 @@ def XXheatmap(
     change_colorbar: bool = True,
     colorbar_ticks = [0, 1, 2], 
     colorbar_label = ["copy loss", "copy neutral", "copy gain"], 
-    colorbar_name = "CNV_states",
+    colorbar_name = "CNV states",
     save_file: bool = False,
+    set_dpi = 150,
     out_file = 'XClone_XXheatmap_test.pdf',
     **kwargs):
     """
@@ -412,7 +420,7 @@ def XXheatmap(
     if save_file == False:
         plt.show()
     else:
-        plt.savefig(out_file, bbox_inches='tight', dpi=600)
+        plt.savefig(out_file, bbox_inches='tight', dpi=set_dpi)
         plt.show()
         plt.close()
 
