@@ -13,9 +13,9 @@ from ._data import resort_mtx_bycell
 
 from ._data import xclonedata
 from ._data import extra_anno
-
+from ._data import exclude_XY_adata
 from ._data import check_RDR, check_BAF
-from ._data import check_RDR_BAF_cellorder
+from ._data import check_RDR_BAF_cellorder, check_data_combine
 
 
 # RDR
@@ -25,7 +25,7 @@ from ._RDR_preprocess import Xdata_RDR_preprocess, gene_length_scale
 from ._RDR_genes import get_markers
 from ._RDR_genes import filter_markers
 
-
+from ._preprocessing import valid_cell, gene_filter
 from ._preprocessing import filter_nulldata, filter_2nulldata, tidy_Xdata
 from ._preprocessing import filter_pre
 from ._preprocessing import filter_obs, filter_features
@@ -42,4 +42,6 @@ from ._utils import annotate_node
 
 from ._efficiency import efficiency_preview
 
-from ._Xdata_manipulation import Xdata_region_selection, Xdata_cell_selection
\ No newline at end of file
+from ._Xdata_manipulation import Xdata_region_selection, Xdata_cell_selection
+
+from .xclone_preprocess_wrap import load_Xdata
\ No newline at end of file
